// Copyright 1996-2004, 2012-2016 by Jon Dart.  All Rights Reserved.

// Extract openings from the book into PGN, for test purposes

#include "board.h"
#include "bookdefs.h"
#include "bhash.h"
#include "chessio.h"
#include "movegen.h"
#include "notation.h"
#include "types.h"
#include "debug.h"
#include "attacks.h"
#include "globals.h"
#include "util.h"

#include <fstream>
#include <iostream>
#include <algorithm>
#include <unordered_map>
#include <vector>
using namespace std;

extern "C"{
#include <ctype.h>
#include <math.h>
};

static unordered_map <uint64_t, int>* hashTable = NULL;

void exploreOpenings(Board &board, int currPly, int maxPly, vector<Move> &moves) 
{
   if (currPly == maxPly) {
      if (hashTable->count(board.hashCode())) {
         return;
      }
      (*hashTable)[board.hashCode()] = 1;

      cout << "[Event \"?\"]" << endl;
      cout << "[Site \"?\"]" << endl;
      cout << "[Date \"2017.28.02\"]" << endl;
      cout << "[Round \"?\"]" << endl;
      cout << "[White \"?\"]" << endl;
      cout << "[Black \"?\"]" << endl;
      cout << "[Result \"*\"]" << endl << endl;
      
      Board b;
      int m = 0;
      for (auto it = moves.begin(); it != moves.end(); it++) {
         if (m%2== 0) cout << 1+(m/2) << ". ";
         Notation::image(b,*it,Notation::SAN_OUT,cout);
         b.doMove(*it);
         cout << ' ';
         ++m;
      }
      cout << '*' << endl;
      return;
   }
   vector< pair<Move,int> > results;
   int n = openingBook.book_moves(board,results);
   
   for (auto it = results.begin(); it != results.end(); it++) {
      pair<Move,int> amove = *it;
      moves.push_back(amove.first);
      BoardState state(board.state);
      board.doMove(amove.first);
      exploreOpenings(board,1+currPly,maxPly,moves);
      board.undoMove(amove.first,state);
      moves.pop_back();
   }
}

int CDECL main(int argc, char **argv)
{
   string book_name;

   Bitboard::init();
   initOptions(argv[0]);
   Attacks::init();
   Scoring::init();
   if (!initGlobals(argv[0], false)) {
      cleanupGlobals();
      exit(-1);
   }
   atexit(cleanupGlobals);

   hashTable =  new unordered_map< uint64_t, int>;

   int maxPly = 20;

   int arg = 1;
   while (arg < argc) {
      if (*argv[arg] == '-') {
         char c = argv[arg][1];
         switch(c) {
            case 'p': /* max ply */
               ++arg;
               maxPly = atoi(argv[arg]);
               if (maxPly == 0) {
                  cerr << "Illegal max ply (-p) value" << endl;
                  exit(-1);
               }
               break;
            default:
               cerr << "Illegal switch: " << c << endl;
               exit(-1);
         }
         ++arg;
      }
      else
         break;
   }

   delayedInit();

   options.book.selectivity = 33;
   Board board;
   vector <Move> moves;
   exploreOpenings(board,0,maxPly,moves);

   delete hashTable;
   
}
